/*
 * Copyright (C) 2012 Chris Burdess <dog@bluezoo.org>
 *
 * Gnome Shell Biff is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Gnome Shell Biff is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Gnome Documents; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

const St = imports.gi.St;
const Main = imports.ui.main;
const GConf = imports.gi.GConf;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Mainloop = imports.mainloop;
const Utils = imports.misc.util;
//const EDataServer = imports.gi.EDataServer; // gir1.2-edataserver-1.2

let gconfclient, checkInterval, count, checking;
let button, extensionPath, buttonLabel, iconRead, iconUnread;
let readCountPid, checkMailPid;

function _openMUA() {
    try {
        //let command = gconfclient.get("/desktop/gnome/url-handlers/mailto/command").get_string().split(' ')[0];
        //Utils.trySpawnCommandLine(command);
        let appinfo = Gio.app_info_get_default_for_type("x-scheme-handler/mailto", false);
        let success = appinfo.launch([], null);
        if (!success) {
            global.log("Error launching default MUA");
        }
    } catch (err) {
        global.log("Can't start default MUA: "+err.message);
    }
}

/*function _checkMail2() {
    try {
        let list = gconfclient.get("/apps/evolution/mail/accounts").get_list(GConf.VALUE_STRING);
        for (var i = 0; i < list.length; i++) {
            let account = EDataServer.Account.new_from_xml(list[i].get_string());
            let name = account.get_string(EDataServer.ACCOUNT_NAME);
            // gir FAIL: no method Account.get_string
            let sourceUrl = account.get_string(EDataServer.ACCOUNT_SOURCE_URL);
        }
    } catch (err) {
        global.log("Can't check mail: "+err.message);
    }
}*/

function _readCount() {
    try {
        let newCount = gconfclient.get("/apps/gnome-biff/count").get_int();
        if (newCount != count) {
            _setMessageCount(newCount);
        }
    } catch (err) {
        global.log("Can't read message count: "+err.message);
    }
    return true;
}

function _checkMail() {
    if (!checking) {
        checking = true;
        try {
            let command = extensionPath + "/check_accounts";
            let [success, pid] = GLib.spawn_async(null, [command], null, GLib.SpawnFlags.SEARCH_PATH | GLib.SpawnFlags.REAP_CHILD, null);
            if (!success) {
                global.log("Error executing check_accounts");
            }
        } catch (err) {
            global.log("Error executing check_accounts:\n"+err.message);
        }
        checking = false;
    }
    return true;
}

function _setMessageCount(newCount) {
    count = newCount;
    if (count > 0) {
        buttonLabel.text = count.toString();
        iconRead.hide();
        iconUnread.show();
        buttonLabel.show();
    } else {
        iconUnread.hide();
        iconRead.show();
        buttonLabel.hide();
    }
}

function init(meta) {
    extensionPath = meta.path;

    count = -1;
    checking = false;
    checkInterval = 20;
    gconfclient = GConf.Client.get_default();
    checkIntervalKey = gconfclient.get("/apps/gnome-biff/check-interval");
    if (checkIntervalKey != null) {
        checkInterval = checkIntervalKey.get_int();
        if (checkInterval < 10) {
            checkInterval = 10; // don't invoke check_accounts too frequently
        }
    }

    button = new St.Bin({ style_class: 'panel-button', reactive: true, can_focus: true, x_fill:true, y_fill: false, track_hover:true });
    iconRead = new St.Icon({ icon_name: 'mail-read-symbolic', icon_size: 16, icon_type: St.IconType.SYMBOLIC, style_class: 'system_status_icon' });
    iconUnread = new St.Icon({ icon_name: 'mail-unread-symbolic', icon_size: 16, icon_type: St.IconType.SYMBOLIC, style_class: 'syatem_status_icon' });
    iconUnread.hide();
    buttonLabel = new St.Label({ style_class: 'button_text', text: '...' });
    var box = new St.BoxLayout();
    box.insert_actor(iconRead, 1);
    box.insert_actor(iconUnread, 1);
    box.insert_actor(buttonLabel, 2);
    button.set_child(box);
    button.connect('button-press-event', _openMUA);
}

function enable() {
    Main.panel._rightBox.insert_actor(button, 0);
    checkMailPid = GLib.timeout_add_seconds(0, checkInterval, _checkMail);
    readCountPid = GLib.timeout_add_seconds(0, 3, _readCount);
}

function disable() {
    Main.panel._rightBox.remove_actor(button);
    if (readCountPid != null) {
        Mainloop.source_remove(readCountPid);
        readCountPid = null;
    }
    if (checkMailPid != null) {
        Mainloop.source_remove(checkMailPid);
        checkMailPid = null;
    }
}
